﻿using System;
using System.Collections.Generic;

namespace Collections_06
{
    public static class Solution
    {
        public static void Test ()
        {
            var manager = new SolutionCustomerManager();
            var firstCustomer = new Customer("Tomas Pouzar");           
            var secondCustomer = new Customer("Franta Konyvka");
            var thirdCustomer = new Customer("Jana Ochotska");

            manager.CreateCustomer(firstCustomer);
            manager.CreateCustomer(secondCustomer);
            manager.CreateCustomer(thirdCustomer);

            manager.StoreOrders(new []
            {
                new Order(3, "Zlate polomacene"),
                new Order(1, "Bohemia bramburky"),
            }, 
            firstCustomer);

            manager.StoreOrders(new[]
            {
                new Order(10, "Krusovice Cerne 0.5l"),
            },
            secondCustomer);

            manager.StoreOrders(new[]
            {
                new Order(2, "Tic Tac"),
                new Order(1, "Mentos freshmint")
            },
            thirdCustomer);

            manager.WriteOutAllData();

            Console.ReadKey();
        }

        
    }

    /// <summary>
    /// Stores customers and orders
    /// in appropriate data structure
    /// </summary>
    public class SolutionCustomerManager
    {
        private readonly Dictionary<Customer, List<Order>> customerData =
            new Dictionary<Customer, List<Order>>();


        public void CreateCustomer(Customer customer)
        {
            if (customer != null && !customerData.ContainsKey(customer))
            {
                customerData.Add(customer, new List<Order>());
            }
        }

        public void StoreOrders(Order[] orders, Customer customer)
        {
            if (orders != null && customerData.ContainsKey(customer))
            {
                // AddRange prida do listu kolekci, ktera implementuje IEnumerable,
                // tedy v tomto pripade vsechny prvky z pole Orders
                customerData[customer].AddRange(orders);
            }
        }

        public void WriteOutAllData()
        {
            foreach (var keyValuePair in customerData)
            {
                Console.WriteLine(keyValuePair.Key);
                Console.WriteLine(string.Join(";", keyValuePair.Value));
                Console.WriteLine("\n");
            }
        }
    }

    /// <summary>
    /// Represents Customer which has unique
    /// identifier and collection of orders
    /// </summary>
    public class Customer
    {
        /// <summary>
        /// create globally unique identifier, more on this topic:
        /// https://msdn.microsoft.com/en-us/library/system.guid.newguid%28v=vs.110%29.aspx?f=255&MSPPError=-2147217396
        /// </summary>
        public Guid Id { get; } = Guid.NewGuid();

        public string Name { get; }

        public Customer(string name)
        {
            Name = name;
        }

        public override int GetHashCode()
        {
            return Id.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            return this.GetHashCode() == obj.GetHashCode();
        }

        public override string ToString()
        {
            return Name;
        }
    }

    /// <summary>
    /// represents order, each customer
    /// can have multiple orders
    /// </summary>
    public class Order
    {
        public int Amount { get; private set; }

        public string GoodsDescription { get; private set; }

        public Order(int amount, string goodsDescription)
        {
            Amount = amount;
            GoodsDescription = goodsDescription;
        }

        public override string ToString()
        {
            return GoodsDescription + $" - {Amount}x";
        }
    }


}
