﻿using System;
using System.Collections.Generic;

namespace Generics_05
{
    public static class Solution
    {
        static void Swap(ref int firstInt, ref int secondInt)
        {
            GenericSwap(ref firstInt, ref secondInt);
        }

        static void Swap(ref string firstString, ref string secondString)
        {
            GenericSwap(ref firstString, ref secondString);
        }

        static void GenericSwap<T>(ref T first, ref T second)
        {
            var temp = first;
            first = second;
            second = temp;
        }

        static void AddManyToCollection<T, U>(T collection, U[] items)
            where T : IList<U>
        {
            if (items != null)
            {
                foreach (var item in items)
                {
                    collection.Add(item);
                }
            }            
        }

        static U[] ToArray<T, U>(T list)
            where T : IList<U>
        {
            U[] array = new U[list.Count];

            for (int i = 0; i < list.Count; i++)
            {
                array[i] = list[i];
            }

            return array;
        }

        static U[] ToArray<U>(IList<U> list)
        {
            U[] array = new U[list.Count];

            for (int i = 0; i < list.Count; i++)
            {
                array[i] = list[i];
            }

            return array;
        }

        public static void SwapUtilTest()
        {
            var firstInt = 1;
            var secondInt = 2;

            var firstString = "Knock, knock.";
            var secondString = "Race condition. Who’s there?";

            Console.WriteLine("Integer swap test:");
            Console.WriteLine($"Before: first: {firstInt}, second: {secondInt}");
            Swap(ref firstInt, ref secondInt);
            Console.WriteLine($"After: first: {firstInt}, second: {secondInt}" + Environment.NewLine);
            Console.ReadKey();

            Console.WriteLine("String swap test:");
            Console.WriteLine($"Before: first: {firstString}, second: {secondString}");
            Swap(ref firstString, ref secondString);
            Console.WriteLine($"After: first: {firstString}, second: {secondString}" + Environment.NewLine);
            Console.ReadKey();

            firstInt = 5;
            secondInt = 10;

            firstString = "str1";
            secondString = "str2";


            Console.WriteLine("Generic swap test 01:");
            Console.WriteLine($"Before: first: {firstInt}, second: {secondInt}");
            GenericSwap(ref firstInt, ref secondInt);
            Console.WriteLine($"After: first: {firstInt}, second: {secondInt}" + Environment.NewLine);

            Console.WriteLine("Generic swap test 02:");
            Console.WriteLine($"Before: first: {firstString}, second: {secondString}");
            GenericSwap(ref firstString, ref secondString);
            Console.WriteLine($"After: first: {firstString}, second: {secondString}" + Environment.NewLine);
            Console.ReadKey();

        }

        public static void CollectionUtilTest()
        {
            var list = new List<String>() {"One"};
            
            Console.WriteLine("AddManyToCollection test:");
            Console.WriteLine("Initial content:");
            WriteAllItems(list);
            AddManyToCollection(list, new [] {"Two", "Three"});
            Console.WriteLine("After Addition:");
            WriteAllItems(list);
            Console.WriteLine();
            Console.ReadKey();

            Console.WriteLine("ToArray test:");
            var array = ToArray<List<string>, string>(list);
            Console.WriteLine("Array content:");
            WriteAllItems(array);
            Console.ReadKey();
        }

        private static void WriteAllItems<T>(IEnumerable<T> collection)
        {
            foreach (var item in collection)
            {
                Console.Write($"{item}, ");
            }
            Console.WriteLine();
        }
    }
}
